
/*
 * Momentum window. May 2017
 * Combines simulated and experimental data.
 * 
 * peaks: 1) 0.482 MeV, 2) 0.976 MeV, 3) 1.682 MeV  
 */

void MomentumWindow(int peak = 3, Double_t xscale = 0.925, Int_t bins = 2000, Double_t emin = 0, Double_t emax = 2000, Bool_t vetoOn = false) {

    TString dir = Form("../../output/scan/e%i/", peak);
    TString expdir = "/home/oliskir/Desktop/f20/data/sorted/new/";

    double xmin;
    double xmax;
    if (peak == 1) {
        xmin = 3;
        xmax = 14;
    }
    else if (peak == 2) {
        xmin = 9;
        xmax = 18;
    }
    else if (peak == 3) {
        xmin = 14;
        xmax = 30;
    }
    
    double ampsPerTesla = 1067;
    double maxAmps = 700; // Danfysik supply
    double calib = ampsPerTesla / maxAmps * 100;  // Bmax (T) => I/Imax (%)
    calib *= xscale; // small correction
    
    cout << "Calibration coeffiecient: " << calib << " %/T" << endl;
    
    const int ncuts = 2;
    double cuts_min[ncuts], cuts_max[ncuts];
    
    if (peak == 1) {
        cuts_min[0] =  200; cuts_max[0] = 550;
        cuts_min[1] =  350; cuts_max[1] = 550;
    }
    else if (peak == 2) {
        cuts_min[0] =  600; cuts_max[0] = 1150;
        cuts_min[1] =  850; cuts_max[1] = 1150;
    }
    else if (peak == 3) {
        cuts_min[0] = 1200; cuts_max[0] = 1950;
        cuts_min[1] = 1550; cuts_max[1] = 1950;
    }
    
    // simulated data
    vector<double> bmax;
    if (peak == 1) {
        bmax = vector<double>{0.030, 0.035, 0.040, 0.045, 0.050, 0.055, 0.060, 0.065, 0.070, 0.075, 0.080, 0.085, 0.090, 0.095};
    }
    else if (peak == 2) {
        bmax = vector<double>{0.070, 0.075, 0.080, 0.085, 0.090, 0.095, 0.100, 0.105, 0.110, 0.115, 0.120, 0.125, 0.130, 0.135};
    }
    else if (peak == 3) {
        bmax = vector<double>{0.110, 0.115, 0.120, 0.125, 0.130, 0.135, 0.140, 0.145, 0.150, 0.155, 0.160, 0.165, 0.170, 0.175};
    }
     
    vector<TString> file;
    for (auto& b : bmax) {
        TString fname = dir + Form("bmax%04d.root", (int)(b*1E4)); 
        file.push_back(fname);
    }

    for (auto&b : bmax) b *= calib;    

    // source activity
    double A0    = 398e3; // Start activity in Bq
    double years = 8.0;   // years since source was made
    double t12   = 31.6;  // half-life
    double A     = A0 * TMath::Exp(- years * TMath::Log(2) / t12);
    cout << "Activity: " << A << " Bq" << endl;
    
    // intensities
    double intensity;
    if      (peak == 1) intensity = (15.62 + 4.38)/1E3;
    else if (peak == 2) intensity = (72.43 + 18.38)/1E3;
    else if (peak == 3) intensity = (0.2522 + 0.0403)/1E3;

    // get histograms
    vector<TH1F*> hist;
    Int_t i=0;
    for (auto& fname : file) {
        TFile * f = new TFile(fname, "READ");
        TString hname = Form("sim%i", i+1);
        hist.push_back(new TH1F(hname, hname, bins, emin, emax));
        TTree * t = (TTree*)f->Get("Detector");

        // Normalization
        Double_t norm = ((TParameter<double>*)(t->GetUserInfo()->FindObject("NORMALIZATION")))->GetVal();
	    // Calculate scaling factor (for 1 minute measurement)
	    Double_t sf = intensity * 60 * A / norm;
        if (i==0) cout << "Simulation normalization factor: " << sf << endl;        

        // veto cut
        TCut cut("");
        if (vetoOn) cut = TCut("EDepVeto == 0");
        
        // draw
        t->Draw("EDepSignal>>"+hname, cut);
        
        // scale
        hist[i]->Scale(sf);
        
        i++;

        cout << fname << " " << hname << endl;
    }

    // do integrals      
    double ymax = 0;
    vector<double> integrals[ncuts];
    cout << " I/I_max (%)    Counts/min    Peak frac. (%)" << endl;
    i=0;
    for (auto& h : hist) {
    
        vector<double> counts;
        for (int j = 0; j < ncuts; j++) {
            counts.push_back(h->Integral(h->FindBin(cuts_min[j]), h->FindBin(cuts_max[j])));
            integrals[j].push_back(counts[j]);

            if (counts[j] > ymax) ymax = counts[j];
        }
        
        double frac = counts[2] / counts[0];
    
        cout << Form("  %.1f", bmax[i]) 
             << Form("    %.0f", counts[0])
             << Form("    %.0f%%", frac*100.) << endl;


        i++;
    }


    //----------------------------------------------------------------------//
    // experimental data
    vector<TString> expfile;
    vector<double> current;
    vector<double> duration;
    
    for (int i = 0; i <= 28; i++) {
        int run = 272 + i;
        expfile.push_back(expdir + Form("s0%i_00.root", run)); 
        current.push_back(i);
        if (i <= 15) duration.push_back(1);
        else duration.push_back(5);
    }

    // get histograms
    vector<TH1F*> exphist;
    i=0;
    for (auto& fname : expfile) {
        TFile * f = new TFile(fname, "READ");
        TString hname = Form("exp%i", i+1);
        exphist.push_back(new TH1F(hname, hname, bins, emin, emax));
        TTree * t = (TTree*)f->Get("data");
        TCut cut("");
        if (vetoOn) cut = TCut("M0 == 1 && M1 <= 1 && E1 < 140 + 0.21*E0");
        t->Draw("E0>>"+hname, cut);
        exphist[i]->Scale(1./duration[i]);
        i++;
        cout << fname << " " << hname << endl;
    }    

    // do exp integrals      
    vector<double> expintegrals[ncuts];
    cout << " I/I_max (%)    Counts/min" << endl;
    i=0;
    for (auto& h : exphist) {
    
        vector<double> counts;
        for (int j = 0; j < ncuts; j++) {
            counts.push_back(h->Integral(h->FindBin(cuts_min[j]), h->FindBin(cuts_max[j])));
            expintegrals[j].push_back(counts[j]);
    
        }
    
        cout << Form("  %.0f%%", current[i]) 
             << Form("    %.0f", counts[0]) << endl;

        i++;
    }
    //----------------------------------------------------------------------//

    // canvas
    TCanvas * c1 = new TCanvas("c1");

	// Legend
	Double_t xl1=.63, yl1=0.5, xl2=xl1+0.22, yl2=yl1+0.25;
    TLegend *leg = new TLegend(xl1, yl1, xl2, yl2);

    // TGraphs
    const int M = ncuts;
    TGraph* gr[M];
    for (int i = 0; i < M; i++) {
        gr[i] = new TGraph(integrals[i].size(), bmax.data(), integrals[i].data());      
    }
    
    // Multigraph
    TMultiGraph *mg = new TMultiGraph();
    for (int j = 0; j < M; j++) {
        gr[j]->SetLineColor(j+1);
        gr[j]->SetLineWidth(2);
        mg->Add(gr[j]);
        leg->AddEntry(gr[j], Form("SIM [%.0f<E<%.0f]", cuts_min[j], cuts_max[j]), "l");
    }

    string sEner;
    if      (peak == 1) sEner = "0.482";
    else if (peak == 2) sEner = "0.976";
    else if (peak == 3) sEner = "1.682";

    mg->SetTitle(Form("Momentum window (%s)", sEner.c_str()));
    mg->Draw("AC");

    // experimental data    
    const int N = ncuts;
    TGraphErrors * expgr[N];
    for (int i = 0; i < N; i++) {
        vector<double> ex, ey;
        for (auto& y : expintegrals[i]) {
            ex.push_back(0);
            ey.push_back(sqrt(y));
        }
        expgr[i] = new TGraphErrors(expintegrals[i].size(), current.data(), expintegrals[i].data(), ex.data(), ey.data());      

        int col = 4;
        if (i == 1) col = 6;
        expgr[i]->SetLineColor(col);
        expgr[i]->SetLineStyle(1+i);
        expgr[i]->SetMarkerColor(col);
        expgr[i]->SetMarkerStyle(20+i);
        expgr[i]->Draw("samePL");

        leg->AddEntry(expgr[i], Form("EXP [%.0f<E<%.0f]", cuts_min[i], cuts_max[i]), "p");
    }

    auto yaxis = mg->GetYaxis();
    yaxis->SetTitle("Counts/minute");
    auto xaxis = mg->GetXaxis();
    xaxis->SetTitle("I/I_{max} (%)");
    xaxis->SetLimits(xmin, xmax);         // along X
    mg->GetHistogram()->SetMaximum(1.3*ymax); // along          
    mg->GetHistogram()->SetMinimum(0);  //   Y     

    leg->Draw("Same");

    // text
   TText *t = new TText(0.8*xmax, 1.15*ymax, Form("multiplier: %.3f", xscale));
   t->SetTextAlign(11);
   t->SetTextColor(1);
   t->SetTextFont(43);
   t->SetTextSize(16);
   t->Draw("same");

    // make pretty    
    gPad->SetLogy(0);
    gPad->SetGrid(0,1);
    gPad->SetTicks();
    
    gStyle->SetOptStat("");
    
    TString fname = "/home/oliskir/Dropbox/Work_ln-s/F20-IGISOL/analysis/figures/";
    fname += Form("momentum_window_e%i.png", peak);
    c1->Print(fname);                  
}
